import os
import numpy as np
import matplotlib.pyplot as plt

from sklearn.inspection import DecisionBoundaryDisplay
import matplotlib.font_manager as fm

myfont = fm.FontProperties(fname="./FiraSans-Regular.ttf")

if False:
    # Figure 1: 1D density ========================================================
    figure = plt.figure(figsize=(8, 5), dpi=300, facecolor="#fafafa")

    plt.clf()
    ax = plt.gca()
    ax.set_facecolor("#fafafa")



    T = np.linspace(-3.5, 3.5, 1001)
    dens_1D = np.exp(-T**2 / 2)

    ax.plot(T, dens_1D, color="b", linewidth=2)

    ax.axis([-3.5, 3.5, -0.3, 1.5])
    ax.set_aspect("equal")

    # Move the left and bottom spines to x = 0 and y = 0, respectively.
    ax.spines[["left", "bottom"]].set_position(("data", 0))
    # Hide the top and right spines.
    ax.spines[["top", "right"]].set_visible(False)

    ax.set_xticks([-3,-2,-1,1,2,3])
    ax.set_yticks([])

    # Draw arrows (as black triangles: ">k"/"^k") at the end of the axes.  In each
    # case, one of the coordinates (0) is a data coordinate (i.e., y = 0 or x = 0,
    # respectively) and the other one (1) is an axes coordinate (i.e., at the very
    # right/top of the axes).  Also, disable clipping (clip_on=False) as the marker
    # actually spills out of the axes.
    ax.plot(1, 0, ">k", transform=ax.get_yaxis_transform(), clip_on=False)
    ax.plot(0, 1, "^k", transform=ax.get_xaxis_transform(), clip_on=False)

    plt.tight_layout()
    plt.show(block=False)

    filename = f"output/gaussian_1D.png"
    os.makedirs(os.path.dirname(filename), exist_ok=True)
    plt.savefig(filename, bbox_inches="tight", transparent=False)






    # Figure 2: Chi distributions ========================================================
    figure = plt.figure(figsize=(8, 5), dpi=300, facecolor="#fafafa")

    plt.clf()
    ax = plt.gca()
    ax.set_facecolor("#fafafa")

    for d, color in [
            (1, "#3347db"), 
            (2, "#5833db"),
            (4, "#9533db"),
            (9, "#db33d5"),
            (16, "#db338d"),
            (25, "#db3333"),
            ]:
        x = np.random.randn(100000, d)
        norms = np.sqrt(np.sum(x**2, axis=1))
        ax.hist(norms, bins=140, range=(0, 7), density=True, color=color, linewidth=2, histtype="step")

    ax.axis([0, 7, 0, 1.2])
    ax.set_aspect("equal")

    # Move the left and bottom spines to x = 0 and y = 0, respectively.
    ax.spines[["left", "bottom"]].set_position(("data", 0))
    # Hide the top and right spines.
    ax.spines[["top", "right"]].set_visible(False)

    ax.set_xticks([1,2,3, 4, 5, 6])
    ax.set_yticks([])

    # Draw arrows (as black triangles: ">k"/"^k") at the end of the axes.  In each
    # case, one of the coordinates (0) is a data coordinate (i.e., y = 0 or x = 0,
    # respectively) and the other one (1) is an axes coordinate (i.e., at the very
    # right/top of the axes).  Also, disable clipping (clip_on=False) as the marker
    # actually spills out of the axes.
    ax.plot(1, 0, ">k", transform=ax.get_yaxis_transform(), clip_on=False)
    ax.plot(0, 1, "^k", transform=ax.get_xaxis_transform(), clip_on=False)

    plt.tight_layout()
    plt.show(block=False)

    filename = f"output/norms.png"
    os.makedirs(os.path.dirname(filename), exist_ok=True)
    plt.savefig(filename, bbox_inches="tight", transparent=False)



    # Figure 3: Rescaled Chi distributions ========================================================
    figure = plt.figure(figsize=(8, 5), dpi=300, facecolor="#fafafa")

    plt.clf()
    ax = plt.gca()
    ax.set_facecolor("#fafafa")

    for d, color in [
            (1, "#3347db"), 
            (2, "#5833db"),
            (4, "#9533db"),
            (9, "#db33d5"),
            (16, "#db338d"),
            (25, "#db3333"),
            ]:
        x = np.random.randn(100000, d)
        norms = np.sqrt(np.sum(x**2, axis=1)) / np.sqrt(d)
        ax.hist(norms, bins=140, range=(0, 3), density=True, color=color, linewidth=2, histtype="step")

    ax.axis([0, 3, 0, 3])
    ax.set_aspect("equal")

    # Move the left and bottom spines to x = 0 and y = 0, respectively.
    ax.spines[["left", "bottom"]].set_position(("data", 0))
    # Hide the top and right spines.
    ax.spines[["top", "right"]].set_visible(False)

    ax.set_xticks([1,2])
    ax.set_yticks([1,2])

    # Draw arrows (as black triangles: ">k"/"^k") at the end of the axes.  In each
    # case, one of the coordinates (0) is a data coordinate (i.e., y = 0 or x = 0,
    # respectively) and the other one (1) is an axes coordinate (i.e., at the very
    # right/top of the axes).  Also, disable clipping (clip_on=False) as the marker
    # actually spills out of the axes.
    ax.plot(1, 0, ">k", transform=ax.get_yaxis_transform(), clip_on=False)
    ax.plot(0, 1, "^k", transform=ax.get_xaxis_transform(), clip_on=False)

    plt.tight_layout()
    plt.show(block=False)

    filename = f"output/norms_rescaled.png"
    os.makedirs(os.path.dirname(filename), exist_ok=True)
    plt.savefig(filename, bbox_inches="tight", transparent=False)





    # Figure 4: Rescaled Chi distributions ========================================================
    figure = plt.figure(figsize=(8, 8), dpi=300, facecolor="#fafafa")

    for d in [ 2, 4, 9, 16, 25, 100]:
        plt.clf()
        ax = plt.gca()
        ax.set_facecolor("#fafafa")

        x = np.random.randn(100000, d)
        norms = np.sqrt(np.sum(x**2, axis=1)) / np.sqrt(d)
        x_2d = x[:,:2]
        x_2d = x_2d / np.sqrt(np.sum(x_2d**2, axis=1))[:,None]
        x_2d = x_2d * norms[:,None]

        ax.hist2d(x_2d[:,0], x_2d[:,1], bins=[100,100], range=[[-2,2],[-2,2]], density=True, cmap="inferno")

        ax.axis([-2, 2, -2, 2])
        ax.set_aspect("equal")
        ax.set_xticks(())
        ax.set_yticks(())


        plt.tight_layout()
        plt.show(block=False)

        filename = f"output/rescaled_2d_{d}.png"
        os.makedirs(os.path.dirname(filename), exist_ok=True)
        plt.savefig(filename, bbox_inches="tight", transparent=False)








    # Figure 5: 2D Gaussian ========================================================
    figure = plt.figure(figsize=(8, 8), dpi=300, facecolor="#fafafa")

    plt.clf()
    ax = plt.gca()
    ax.set_facecolor("#fafafa")


    x = np.linspace(-2.0,2.0,101)
    y = np.linspace(-2.0,2.0,101)
    X,Y = np.meshgrid(x, y) # grid of point
    Z = X**2 + Y**2
    Z = np.exp(-Z)

    im = ax.imshow(Z,cmap="inferno") # drawing the function

    ax.set_aspect("equal")
    ax.set_xticks(())
    ax.set_yticks(())


    plt.tight_layout()
    plt.show(block=False)

    filename = f"output/gaussian_2d.png"
    os.makedirs(os.path.dirname(filename), exist_ok=True)
    plt.savefig(filename, bbox_inches="tight", transparent=False)




# Figure 6+: White noise ===============================================================


figure = plt.figure(figsize=(8, 8), dpi=300, facecolor="#fafafa")

plt.clf()
ax = plt.gca()
ax.set_facecolor("#fafafa")

X = np.random.randn(10, 10)
Y = np.random.randn(10, 10)



im = ax.imshow(0*X,cmap="coolwarm", vmin=-3, vmax=3) # drawing the function

ax.set_aspect("equal")
ax.set_xticks(())
ax.set_yticks(())

plt.tight_layout()
plt.show(block=False)

filename = f"output/white_noise_zero.png"
plt.savefig(filename, bbox_inches="tight", transparent=False)




im = ax.imshow(X,cmap="coolwarm", vmin=-3, vmax=3) # drawing the function

ax.set_aspect("equal")
ax.set_xticks(())
ax.set_yticks(())

plt.tight_layout()
plt.show(block=False)

filename = f"output/white_noise_1.png"
plt.savefig(filename, bbox_inches="tight", transparent=False)



im = ax.imshow(Y,cmap="coolwarm", vmin=-3, vmax=3) # drawing the function

ax.set_aspect("equal")
ax.set_xticks(())
ax.set_yticks(())

plt.tight_layout()
plt.show(block=False)

filename = f"output/white_noise_2.png"
plt.savefig(filename, bbox_inches="tight", transparent=False)




im = ax.imshow(X-Y,cmap="coolwarm", vmin=-3, vmax=3) # drawing the function

ax.set_aspect("equal")
ax.set_xticks(())
ax.set_yticks(())

plt.tight_layout()
plt.show(block=False)

filename = f"output/white_noise_diff.png"
plt.savefig(filename, bbox_inches="tight", transparent=False)


