---
title: "AI for healthcare"
author: "Jean Feydy"
institute: ""
topic: ""
theme: "metropolis"
colortheme:
fonttheme:
fontsize: 11pt
linkcolor: black
urlcolor: blue
linkstyle: bold
aspectratio: 169
titlegraphic: 
logo:
date:
section-titles: false
toc: false
---


## Recap of the first two lectures

Lecture 1 -- **Introduction**:

- AI = model + data.
- A good model is simple, accurate and **honest**.
- Understanding your model is key to **creativity**.

 \
Lecture 2 -- **Flat** vector spaces:

- Talk to domain experts $~~\heartsuit$
- Best-case scenario: high-quality, informative features.
- Well-understood **baselines**: trees, K-NNs, linear and kernel regression.




## What about the curse of dimensionality?

`\begin{center}\vspace*{.5cm}`{=latex}

![](images/dataset/dataset_original.pdf){ height=65% }

Remember: machine learning is about tables \
that have **more columns than rows**.

`\end{center}`{=latex}



## The statistical curse of dimensionality (XKCD 882)

`\begin{center}`{=latex}

![](images/XKCD_882/1.png){ height=97% }

`\end{center}`{=latex}

## The statistical curse of dimensionality (XKCD 882)

`\begin{center}`{=latex}

![](images/XKCD_882/2.png){ height=97% }

`\end{center}`{=latex}

## The statistical curse of dimensionality (XKCD 882)

`\begin{center}`{=latex}

![](images/XKCD_882/3.png){ height=97% }

`\end{center}`{=latex}

## The statistical curse of dimensionality (XKCD 882)

`\begin{center}`{=latex}

![](images/XKCD_882/4.png){ height=97% }

`\end{center}`{=latex}


## The statistical curse of dimensionality `\cite{spurious_correlations}`{=latex}

`\begin{center}`{=latex}

![](images/spurious_correlation.png){ width=85% }

`\end{center}`{=latex}

Two simple workarounds:

- **Sparsity**: trees, lasso...
- **Smoothness**: polynomial regression, kernels...



## What does a Normal distribution look like... in dimension 1?


`\begin{center}\vspace*{.5cm}`{=latex}

![](images/gaussians/gaussian_1D.png){ width=85% }

`\end{center}`{=latex}


## What does a Normal distribution look like... in dimension 2?


`\begin{center}\vspace*{.5cm}`{=latex}

![](images/gaussians/gaussian_2d.png){ height=85% }

`\end{center}`{=latex}


## What does a Normal distribution look like... in higher dimension?


`\begin{center}\vspace*{.5cm}`{=latex}

![](images/gaussians/norms_1d.pdf){ width=95% }

**Histograms** for 100,000 points x with D features of the Euclidean norm:
$$
\|\text{x}\| ~=~ \sqrt{\text{x}[\text{1}]^2 + \cdots + \text{x}[\text{D}]^2}~.
$$
We recognize the **sum of D independent**, indentically distributed variables of **mean 1**. \
Taking the square root, we get a random variable of mean $\sqrt{\text{D}}$.

`\end{center}`{=latex}



## What does a Normal distribution look like... in higher dimension?


::: columns
:::: {.column width=50%}
`\begin{center}\vspace*{1cm}`{=latex}

**Histograms** for 100,000 points x with D features of the scaled Euclidean norm:

$$
\frac{\text{1}}{\sqrt{\text{D}}}\|\text{x}\| ~=~ \sqrt{\tfrac{\text{1}}{\text{D}}\big( \text{x}[\text{1}]^2 + \cdots + \text{x}[\text{D}]^2 \big) }~.
$$

 \
As predicted by the **central limit theorem**, $\frac{\text{1}}{\sqrt{\text{D}}}\|\text{x}\|$
concentrates around its mean value 1.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}\vspace*{.5cm}`{=latex}

![](images/gaussians/rescaled_1d.pdf){ height=70% }


`\end{center}`{=latex}
::::
:::



## The "soap bubble" effect (Cf. Ferenc Huszár)



::: columns
:::: {.column width=50%}
`\begin{center}\vspace*{1cm}`{=latex}


**Histograms** for 100,000 points x with D features of the rescaled planar projections:

$$
\frac{\|\text{x}\|}{\sqrt{\text{D}} \cdot \sqrt{\text{x}[\text{1}]^\text{2} + \text{x}[\text{2}]^\text{2}} } \big( \text{x}[\text{1}], \text{x}[\text{2}] \big)~.
$$

 \
This provides a **faithful** visualization of a Normal Gaussian sample in dimension D.


`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}\vspace*{.5cm}`{=latex}

![](images/gaussians/rescaled_2d.pdf){ height=70% }


`\end{center}`{=latex}
::::
:::



## High-dimensional i.i.d. samples = white noise



::: columns
:::: {.column width=23%}
`\begin{center}\vspace*{.5cm}`{=latex}
![](images/gaussians/white_noise_zero.png){ height=3cm }

Mean value.

`\end{center}`{=latex}
::::
:::: {.column width=23%}
`\begin{center}\vspace*{.5cm}`{=latex}
![](images/gaussians/white_noise_1.png){ height=3cm }

Sample $\text{x}_\text{1}$.

`\end{center}`{=latex}
::::
:::: {.column width=23%}
`\begin{center}\vspace*{.5cm}`{=latex}
![](images/gaussians/white_noise_2.png){ height=3cm }

Sample $\text{x}_\text{2}$.

`\end{center}`{=latex}
::::
:::: {.column width=23%}
`\begin{center}\vspace*{.5cm}`{=latex}
![](images/gaussians/white_noise_diff.png){ height=3cm }

Difference $\text{x}_\text{1} - \text{x}_\text{2}$.

`\end{center}`{=latex}
::::
:::



`\begin{center}`{=latex}

 \
Samples do **not** look like the average value of the distribution, \
and are all orthogonal to each other.

`\end{center}`{=latex}


## Geometric consequences of the curse of dimensionality

If we assume that our $\text{D} > \text{10}$ features are `\BR{independent}`{=latex} and identically distributed:

- We require $\text{10}^\text{D}$ samples to enable basic statistics: histograms, density estimation...
- $\|\text{x}_\text{i} - \text{x}_\text{j}\|$ is **constant** up to a minor deviation.
- The distance matrix contains no useful information.
- All of **our intuitions break down**.


## White noise is useless: garbage in, garbage out (XKCD 1838)

`\begin{center}`{=latex}

![](images/machine_learning.png){ height=97% }

Doing machine learning is about understanding that \
your data is **not** white noise. \
\
We need to put a **low-dimensional** structure on our problem.

`\end{center}`{=latex}


## The manifold hypothesis

`\begin{center}`{=latex}

![](images/swiss_roll.png){ height=70% }

What matters for statistics is the **intrinsic** dimension "d" of the dataset, \
not the **extrinsic** dimension "D" of the feature space.

`\end{center}`{=latex}


## Overview of the class

Coming next:

- Lecture 3: **"Discrete"** geometries = graphs.
- Lecture 4: Deep learning on graphs and point clouds.
- Lecture 5: **"Continuous"** geometries = manifolds.
- Lecture 6: Spaces of probability distributions.
- Lecture 7: **Hardware** bottlenecks.

`\begin{center}`{=latex}

 \
The aim of the class is to let you bridge the gap between \
"discrete" and "continuous" `\BR{descriptions}`{=latex} \
of the underlying problem structures.
`\end{center}`{=latex}


## Today's lecture


**1. Why do we care about graphs?**

`\vspace{.25cm}`{=latex}

**2. Local descriptors and archetypes:**

  - Dimension.
  - Curvature.

`\vspace{.25cm}`{=latex}

**3. Global embeddings:**

  - Lab session with UMAP.


# Why do we care about graphs?

## Yet another cake with our domain experts...

`\begin{center}\vspace*{.5cm}`{=latex}

![](images/cakes/cakes.png){ width=95% }

 \
"We may not fully understand the columns of our table... \
But we can certainly tell you that Patient A **is similar** to Patient B!"

`\end{center}`{=latex}



## Undirected graphs

`\begin{center}\vspace*{.5cm}`{=latex}

![](images/6n-graf.pdf){ height=60% }

**Algorithmic** definition: a collection of vertices and edges. \
**Geometric** perspective: a metric space that is defined **locally**... \
and that we would like to understand **globally**!

`\end{center}`{=latex}


## Networks and webs

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}
![](images/plan-metro.png){height=5cm}

Transportation networks.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}
![](images/electric_wires.jpg){height=5cm}

Communication and power lines.

`\end{center}`{=latex}
::::
:::



## 3D curves and meshes `\cite{fischl2012freesurfer}`{=latex}

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}
![](images/arteries.png){height=5cm}

Vascular networks.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}
![](images/freesurfer.jpeg){height=5cm}

Anatomical surfaces.

`\end{center}`{=latex}
::::
:::


`\begin{center}\vspace*{.5cm}`{=latex}

**Intrinsic** graph distances $~~\neq~~$
**Extrinsinc** Euclidean distances.

`\end{center}`{=latex}



## From high-dimensional samples to graphs: K-Nearest Neighbors `\cite{peyre2011numerical}`{=latex}

`\begin{center}\vspace*{.5cm}`{=latex}

![](images/knn_graph.png){ height=60% }

If your data has a low-dimensional structure, \
this should be visible on its **neighborhood** structure.

`\end{center}`{=latex}



## From high-dimensional samples to graphs: kernel matrices

`\begin{center}\vspace*{.5cm}`{=latex}

![](images/kernel_connectivity/connectivity_matrix.pdf){ height=85% }


`\end{center}`{=latex}



## Classical geometry `\cite{konigsberg,albert_chern}`{=latex}


::: columns
:::: {.column width=55%}
`\begin{center}`{=latex}
![](images/konigsberg.png){height=4cm}

 \
Discrete graph.

`\end{center}`{=latex}
::::
:::: {.column width=5%}
```{=latex}
\begin{center}
\vspace*{1.55cm}
{\LARGE $\neq$}
\end{center}
```

::::
:::: {.column width=40%}
`\begin{center}`{=latex}
![](images/albert_chern.jpg){height=4cm}

 \
Continuous surface.

`\end{center}`{=latex}
::::
:::



## Modern geometry `\cite{hyperbolic_salad}`{=latex}


::: columns
:::: {.column width=47%}
`\begin{center}`{=latex}
![](images/Cayley_graph_of_F2.pdf){height=4cm}

 \
Tree.

`\end{center}`{=latex}
::::
:::: {.column width=5%}
```{=latex}
\begin{center}
\vspace*{1.7cm}
{\LARGE $\simeq$}
\end{center}
```

::::
:::: {.column width=47%}
`\begin{center}`{=latex}
![](images/lettuce.jpg){height=4cm}

 \
Hyperbolic salad.

`\end{center}`{=latex}
::::
:::


## It is not the encoding that matters -- but the geometry that is inside `\cite{sharp2021diffusion}`{=latex}


`\begin{center}\vspace*{.5cm}`{=latex}

![](images/diffusionnet_teaser.jpg){ height=60% }

Mesh triangulations, sampling densities or point cloud representations \
should **not** distract ourselves from the underlying objects.

`\end{center}`{=latex}


## But how do we **untangle** such a web of vertices and edges? `\cite{spaghetti}`{=latex}


`\begin{center}\vspace*{.5cm}`{=latex}

![](images/spaghetti.jpg){ height=70% }

Going from `\BR{local}`{=latex} connectivity to `\BR{global}`{=latex} structure \
is the main open challenge in geometry.

`\end{center}`{=latex}


# Dimension

## Dimension = number of degrees of freedom


::: columns
:::: {.column width=30%}
`\begin{center}`{=latex}
![](images/Pong.png){height=4cm}

 \
Pong is 1D.

`\end{center}`{=latex}
::::
:::: {.column width=30%}
`\begin{center}`{=latex}
![](images/Pac-man.png){height=4cm}

 \
Pac-Man is 2D.

`\end{center}`{=latex}
::::
:::: {.column width=35%}
`\begin{center}`{=latex}
![](images/minecraft.png){height=4cm}

 \
Minecraft is 3D.

`\end{center}`{=latex}
::::
:::


## What about geometric objects?


`\begin{center}`{=latex}
![](images/dimension/dimension_ball.pdf){height=6cm}

`\end{center}`{=latex}


## Fitting an ellipsoid to a K-NN graph is easy using local PCA


`\begin{center}\vspace*{1cm}`{=latex}
![](images/dimension/local_PCA.pdf){height=6cm}

`\end{center}`{=latex}




## What is the dimension of an ellipsoid?

`\begin{center}`{=latex}
![](images/dimension/dimension_ellipsoid.pdf){height=6cm}

`\end{center}`{=latex}


## A rule of thumb


`\begin{center}`{=latex}

Let $\lambda_\text{1}, \dots, \lambda_\text{D}$ denote the lengths of the principal axes. \
\
$\lambda_\text{1}^\text{2} > \dots > \lambda_\text{D}^\text{2}$ are the diagonal coefficients of the PCA. \
\
We normalize them as $~~\text{l}_\text{i} = \lambda_\text{i}^\text{2} / (\lambda_\text{1}^\text{2} + \dots + \lambda_\text{D}^\text{2})$.

Then, we may define the **local dimension** d as \
the smallest index such that \
$\text{l}_\text{1} + \cdots + \text{l}_\text{d} > \text{80\%}$.\
\
Other conventions exist!

`\end{center}`{=latex}

## What about "pure" graphs?


::: columns
:::: {.column width=50%}
`\begin{center}\vspace*{1cm}`{=latex}



Hausdorff dimension: we pick $\text{d}$ such that:\
 \
$\mathrm{Vol}(B(\text{x},\text{r})) \sim \text{r}^\text{d}$.\
 \
We estimate:

$$\text{d} \simeq \frac{\log(\mathrm{Vol}(B(\text{x},\text{r})))}{\log(\text{r})}$$

by a linear regression on $\text{r} = \text{1}, \text{2}, \text{3}, \text{4}, \text{5}, \ldots$



`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}\vspace*{.5cm}`{=latex}

![](images/dimension/ball_grid.pdf){height=6cm}


`\end{center}`{=latex}
::::
:::




## This definition works well for many objects, including fractals.

`\vspace*{.5cm}`{=latex}

::: columns
:::: {.column width=30%}
`\begin{center}`{=latex}
![](images/dimension/square_dim.pdf){height=4cm}

 \
$\text{d} = \frac{\log{\text{4}}}{\log{\text{2}}} = \text{2}$

`\end{center}`{=latex}
::::
:::: {.column width=30%}
`\begin{center}`{=latex}
![](images/dimension/cubic_dim.pdf){height=4cm}

 \
$\text{d} = \frac{\log{\text{8}}}{\log{\text{2}}} = \text{3}$

`\end{center}`{=latex}
::::
:::: {.column width=35%}
`\begin{center}`{=latex}
![](images/dimension/fractal_dim.pdf){height=4cm}

 \
$\text{d} = \frac{\log{\text{3}}}{\log{\text{2}}}$

`\end{center}`{=latex}
::::
:::


## Problem: $\mathrm{Vol}(B(\text{x},\text{r}))$ is not always a polynomial function of $\text{r}$ `\cite{topping2021understanding}`{=latex}


::: columns
:::: {.column width=30%}
`\begin{center}`{=latex}
![](images/dimension/clique.png){height=4cm}

 \
**Cliques**: we fill the graph \
and **plateau** very quickly.

`\end{center}`{=latex}
::::
:::: {.column width=30%}
`\begin{center}`{=latex}
![](images/dimension/grid.png){height=4cm}

 \
**Grids**: we retrieve a **polynomial**.

`\end{center}`{=latex}
::::
:::: {.column width=30%}
`\begin{center}`{=latex}
![](images/dimension/tree.png){height=4cm}

 \
**Trees**: the volume of a ball \
grows **exponentially** fast.

`\end{center}`{=latex}
::::
:::

`\begin{center}`{=latex}
This reminds us of **classical examples** in continuous geometry:\
the sphere, the Euclidean plane and the Poincaré disk.
`\end{center}`{=latex}
# Curvature


## Curvature of a 2D surface

`\begin{center}`{=latex}
![](images/curvature/principal_curvatures.png){height=95%}
`\end{center}`{=latex}


## Direct uses in physics and biology

`\begin{center}`{=latex}
![](images/biology/soap.png){height=4cm}
`\end{center}`{=latex}

Soap bubbles minimize:
$$\mathrm{area(\mathcal{S})} = \int_{\mathcal{S}} 1 \, dA $$
under constant volume, or with boundary conditions. \
They correspond to minimal surfaces with $~~H = \kappa_1 + \kappa_2 = 0~~$ in cases 2 and 3.


## Direct uses in physics and biology

`\begin{center}`{=latex}
![](images/biology/RBC.png){height=4cm}
`\end{center}`{=latex}

Red blood cells minimize:
$$\mathrm{Helfrich}(\mathcal{S}) = \int_{\mathcal{S}} (H-H_0)^2 \, dA$$
or a variant of this energy, under constant volume.


## Direct uses in physics and biology `\cite{song2022generation}`{=latex}

`\begin{center}`{=latex}
![](images/biology/curvatubes.png){height=4cm}
`\end{center}`{=latex}

Curvatubes minimize:
$$F(\mathcal{S}) = \int_{\mathcal{S}} p(\kappa_1,\kappa_2) \, dA$$
under constant volume, where: 
$$p(\kappa_1,\kappa_2) = a_{2,0} \, \kappa_1^2 + a_{1,1} \, \kappa_1 \kappa_2 + a_{0,2} \, \kappa_2^2 + a_{1,0} \,\kappa_1 + a_{0,1} \,\kappa_2 + a_{0,0}~.$$


## Curvature is a powerful descriptor

`\begin{center}\vspace*{.5cm}`{=latex}
![](images/biology/simus.png){width=100%}
`\end{center}`{=latex}


## Sidenote: your skillset goes **way beyond** deep learning research

`\begin{center}`{=latex}
![](images/biology/surface_phase.png){height=3.5cm}
`\end{center}`{=latex}

An **inspiring** model:

- Surface energy $~\to~$ convolutional volumetric loss function (phase-field).
- Start with white noise (texture generation) and minimize with gradient descent.
- Implemented on GPU with PyTorch.

`\begin{center}`{=latex}
Combines maths + GPU computing + imaging data $\Longrightarrow$ Perfectly within your reach!
`\end{center}`{=latex}


## What about graphs? `\cite{topping2021understanding}`{=latex}


`\begin{center}`{=latex}
![](images/curvature/geodesics.png){height=5cm}
`\end{center}`{=latex}

**Theorem Egregium** (Gauss, 1827):

- $H = \kappa_1 + \kappa_2$ is extrinsic $~~\to~~$ depends on the embedding.
- $K = \kappa_1 ~\,\cdot~\, \kappa_2$ is intrinsic $~~\to~~$ can be defined on graphs.


## Gromov's hyperbolicity `\cite{gromov1987hyperbolic}`{=latex}


::: columns
:::: {.column width=45%}
`\begin{center}`{=latex}
![](images/curvature/gromov_hyperbolic.png){height=5cm}

 \
A graph is $\delta$-hyperbolic if \
all geodesic triangles are **thin**.

`\end{center}`{=latex}
::::
:::: {.column width=45%}
`\begin{center}`{=latex}
![](images/curvature/SL2_Z_u,s,-id_rayon8.dot-neato-point.pdf){height=5cm}

 \
Global definition, suited to the study \
of groups such as $\text{SL}_\text{2}(\mathbb{Z})$.

`\end{center}`{=latex}
::::
:::




## Forman curvature `\cite{sreejith2016forman,samal2018comparative,topping2021understanding}`{=latex}


`\begin{center}`{=latex}
![](images/curvature/Forman.png){height=5cm}
`\end{center}`{=latex}

**Forman** curvature of an **edge** $\text{i}\leftrightarrow \text{j}~$:
$$4 - \text{degree}_\text{i} - \text{degree}_\text{j} + 3 \cdot \text{triangles}(\text{i}, \text{j},\,\cdot\,)~.$$ 
More precise but complex variations of this formula also exist.



## Ollivier-Ricci curvature `\cite{wang2022applying}`{=latex}

`\begin{center}`{=latex}
![](images/curvature/Ollivier.png){height=5cm}

**Ollivier-Ricci** curvature of an **edge** $\text{i}\leftrightarrow \text{j}~$: \
is the optimal transport distance between $\mathcal{N}(\text{x}_\text{i}, 1)$ and $\mathcal{N}(\text{x}_\text{j}, 1)$ \
larger than the distance between $\text{x}_\text{i}$ and $\text{x}_\text{j}$?
`\end{center}`{=latex}


## Toy models

::: columns
:::: {.column width=30%}
`\begin{center}`{=latex}
![](images/curvature/5_triangles.jpg){height=4cm}

 \
5 triangles per vertex:\
**positive** curvature.

`\end{center}`{=latex}
::::
:::: {.column width=30%}
`\begin{center}`{=latex}
![](images/curvature/6_triangles.jpg){height=4cm}

 \
6 triangles per vertex:\
**flat** curvature.

`\end{center}`{=latex}
::::
:::: {.column width=30%}
`\begin{center}`{=latex}
![](images/curvature/7_triangles.jpg){height=4cm}

 \
7 triangles per vertex:\
**negative** curvature.

`\end{center}`{=latex}
::::
:::

`\begin{center}`{=latex}

 \
$\Longrightarrow~~$ Basic intuition that guides current research in the field.
`\end{center}`{=latex}



# References

